/* ------------------
 * Logic2CNF
 * $Id: grammar.l 504 2010-06-10 15:01:13Z eb771 $
 *
 * Copyright (c) 2010, Edd Barrett (University of Kent)
 * <eddbarrett at googlemail dot com>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

%{

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "logic2cnf.h"
#include "y.tab.h"

/* not using the built in one because it makes noise about unused funcs */
int	my_yylineno = 1; 
char	*err_tmp;	/* temproary buffer for building a yyerror string */

%}

%option nounput
%option noyywrap

%%

[\t ]		{}
"def"		{ return DEF; }
"uq_elim"	{ return UQ_ELIM; }
"resolve"	{ return RESOLVE; }
"report"	{ return REPORT_LITERALS; }
"report_dec"	{ return REPORT_DEC; }
"find_max"	{ return FIND_MAX; }
"find_min"	{ return FIND_MIN; }
"."		{ return AND; }
"+"		{ return OR; }
"~"		{ return NOT; }
";"		{ return END; }
"<=>"		{ return BIMPL; }
"<="		{ return LIMPL; }
"=>"		{ return RIMPL; }
"@"		{ return XOR; }
"#".*		{ } /*comment*/
-[^-]*-		{ yylval.chars = strdup(yytext); return REP_CMMT; }
[a-z][a-zA-Z0-9_]*	{ yylval.chars = strdup(yytext); return LIT; }
[0-9]*		{ yylval.num = atol(yytext); return NUM; }
"("		{ return LBRA; }
")"		{ return RBRA; }
"\n"		{ my_yylineno ++; }
.		{
			lerr(1, "%s line %d:\n\tunexpected token '%s'",
			    yyin_filename, my_yylineno, yytext);
		}

%%
