#!/bin/sh
# Regression testing for logic2cnf

OUTDIR=output
EXPECTEDDIR=expected
CASEDIR=cases
PWD=`pwd`

if [ `basename ${PWD}` != "tests" ]; then
	echo "*error: you must run the regression tests from within the 'tests'
	directory"
	exit 1
fi

if [ ! -d ${OUTDIR} ]; then
	mkdir ${OUTDIR}
fi

TOT_TESTS=0
TESTS_OK=0
TESTS_FAILED=0

TESTS=`ls ${CASEDIR} | grep -e '\.l2c$'`

for TEST in ${TESTS}; do

	TOT_TESTS=`expr ${TOT_TESTS} + 1`

	TEST_COUNT=`grep -e '^# TEST|.*$' ${CASEDIR}/${TEST} | \
	    wc -l | tr -d ' '`
	
	if [ "${TEST_COUNT}" != "1" ]; then
		echo "*error: Test '${TEST}' has wrong number of '# TEST|' lines: '${TEST_COUNT}'"
		echo "		[ FAIL ]"
		TESTS_FAILED=`expr ${TESTS_FAILED} + 1`
		continue
	fi

	EXEC=`grep -e '^# TEST|.*$' ${CASEDIR}/${TEST}`
	EXPECTED=`echo "expected/${TEST}" | sed -e 's/\.l2c$/.out/g'`
	OUT=`echo "output/${TEST}" | sed -e 's/\.l2c$/.out/g'`

	echo  "Running test '${TEST}'... "

	EXEC_EXPAND=`echo ${EXEC} | sed -e "s/@@THIS@@/${CASEDIR}\/${TEST}/g" \
	    -e "s/^# TEST|//g"`

	# execute and catch all badness, if any.
	sh -c "${EXEC_EXPAND} > $OUT 2>&1"

	if [ ! -f ${EXPECTED} ]; then
		echo "*error: no expected outcome for '${TEST}'"
		echo "		[ FAIL ]"
		TESTS_FAILED=`expr ${TESTS_FAILED} + 1`
		continue
	fi	

	OK=`diff -u ${EXPECTED} ${OUT} | wc -l | tr -d ' '`
	if [ "${OK}" = "0" ]; then
		echo "		[ OK ]"
		TESTS_OK=`expr ${TESTS_OK} + 1`
	else
		echo "		[ FAIL ]"
		echo "---------------8<---------------"
		diff -u ${EXPECTED} ${OUT}
		echo "---------------8<---------------"
		TESTS_FAILED=`expr ${TESTS_FAILED} + 1`
	fi
done

echo
echo "REGRESSION TEST SUMMARY"
echo "========================"
echo "Total  = ${TOT_TESTS}"
echo "Passed = ${TESTS_OK}"
echo "Failed = ${TESTS_FAILED}"
echo "========================"

exit ${TESTS_FAILED}
