/*
 * A Portable asprintf() implementation under ISC License
 *
 * Copyright (c) 2010, Edd Barrett <e.barrett@kent.ac.uk>
 * $Id: util.c 504 2010-06-10 15:01:13Z eb771 $
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "config.h"
#include "logic2cnf.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

void *
xmalloc(size_t sz)
{
	void *ptr = malloc(sz);
	if (ptr == NULL) {
		free_everything();
                lerr(1, "xmalloc():\n\t%s", strerror(errno));
        }

        return ptr;
}

void *
xrealloc(void *ptr, size_t sz)
{
	ptr = realloc(ptr, sz);
	if (ptr == NULL) {
		free_everything();
                lerr(1, "xrealloc():\n\t%s", strerror(errno));
        }

        return ptr;
}

#ifndef HAVE_ASPRINTF
#warning WARNING! Using replacement asprintf()
#define ASPRINTF_TMP_TPL	"tmp.asprintf.XXXXXXXX"

int
asprintf(char **ret, const char *fmt, ...)
{
	int	sz;
	va_list	ap;
	char	*buf = (char *) xmalloc(1); /* cast for c++ */

	va_start(ap, fmt);

	/* get size needed */
	sz = vsnprintf(buf, 1, fmt, ap);
	if (sz == -1)
		return -1;

	buf = (char *) xrealloc(buf, sz + 1);
	sz = vsnprintf(buf, sz + 1, fmt, ap);

	va_end(ap);

	*ret = buf;
	return sz;
}
#endif /* HAVE_ASPRINTF */

